#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

int main(int argc, char** argv){
	//counters
	int i, inc, outc, bytec, bytet, percc;
	
	//this is a boolean flag that is set if the user specified
	//double precision output on the arg list
	int doubleFlagSet = 0;
	int silentFlagSet = 0;
	
	//placeholder for parsed double
	double tmpd;
	float tmpf;
	
	//seek and count variables
	int seek = -1, count = -1;
	
	//format string
	char* format = "%f ";
	int linelength = -1;

	//set in and out
	FILE *in, *out;

	//default in and out streams
	in = stdin;
	out = stdout;
	bytet = -1; //we dont know the length of stdin
	
	/** read user params */
	
	for(i=0; i < argc; i++){
		//check for help flag
		if(strcmp(argv[i], "help") == 0 || strcmp(argv[i], "-help") == 0){
			printf("USAGE fbinread [-format STR] [-linelength N] [-if infile] [-of outfile] \n");
			printf("               [-single] [-double] [-seek] [-count] [-help]\n");
			printf("\n");
			printf("Reads floating point data from stdin (or infile), formats it and writes it to \n");
			printf("stdout (or outfile). Formatting string can be specified. Default is \"%%f \" \n");
			printf("\n");
			printf("If linelength is given, a \\n is added after that number of elements \n");
			printf("\n");
			printf("Default precision is single.\n");
			printf("\n");
			return ;
		}
		//check for single
		if(strcmp(argv[i], "-single") == 0){
			doubleFlagSet = 0;
		}
		//check for double
		if(strcmp(argv[i], "-double") == 0){
			doubleFlagSet = 1;
		}
		//check for silent
		if(strcmp(argv[i], "-silent") == 0){
			silentFlagSet = 1;
		}
		//check for if
		if(strcmp(argv[i], "-if") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -if flag with no input file.\n");
				return ;
			}
			//try to open the file
			if( (in=fopen(argv[++i],"r")) == NULL ){
				printf("Error: could not open %s for reading.\n",argv[i]);
				return ;
			}
			//get length of file
			fseek(in, 0, SEEK_END);
			bytet = ftell(in);
			fseek(in, 0, SEEK_SET);
		}
		//check for of
		if(strcmp(argv[i], "-of") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -of flag with no output file.\n");
				return ;
			}
			//try to open the file
			if( (out=fopen(argv[++i],"wb")) == NULL ){
				printf("Error: could not open %s for writing.\n",argv[i]);
				return ;
			}
		}
		//check for format
		if(strcmp(argv[i], "-format") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -format flag with no format.\n");
				return ;
			}
			format = argv[++i];
		}
		//check for linelength
		if(strcmp(argv[i], "-linelength") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -linelength flag with no linelength.\n");
				return ;
			}
			linelength = atoi(argv[++i]);
		}
		//check for seek
		if(strcmp(argv[i], "-seek") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -seek flag with no seek.\n");
				return ;
			}
			seek = atoi(argv[++i]);
		}
		//check for count
		if(strcmp(argv[i], "-count") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -count flag with no count.\n");
				return ;
			}
			count = atoi(argv[++i]);
		}
	}
	
	/** end reading user params */
	
	//keep a line and char counter just for nice error messages
	inc = outc = bytec = percc = 0;

	//read until EOF
	while(1){
		
		//read data
		if(!doubleFlagSet){
			if( fread(&tmpf, sizeof(float), 1, in) != 1 ){
				break ;
			}		
		}
		else{
			if( fread(&tmpd, sizeof(double), 1, in) != 1 ){
				break ;
			}
		}
		
		//increment byte counter
		bytec += doubleFlagSet ? sizeof(double) : sizeof(float);
		
		//print out a status update
		if(!silentFlagSet){
			while(bytet > 0 && bytec*20/bytet > percc){
				fprintf(stderr, "%d%% complete\n", percc*5);
				percc++;
			}
		}

		//increment the input counter
		inc++;

		//write data
		//check to see if we've already printed too many
		if(count >= 0 && outc >= count){
			break ;
		}
		//check to see if we havent skipped enough
		else if(seek >= 0 && inc <= seek){
			continue ;
		}
		else if(!doubleFlagSet){
			fprintf(out, format, tmpf);
			outc++;
		}
		else{
			fprintf(out, format, tmpd);
			outc++;
		}

		//keep a count of how many items written

		//write a newline if needed
		if(linelength > 0 && outc % linelength == 0)
			fprintf(out, "\n");
	
	}
	
	//good manners - close things
	if(in != stdin) fclose(in);
	if(out != stdout) fclose(out);

	//done
	if(!silentFlagSet){
		fprintf(stderr, "Done! Wrote %d floats.\n", outc);
	}
}
