#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#define BUFFSIZE 256

int main(int argc, char** argv){
	//character buffer - it is one char larger than buff size
	//so it can hold a \0 line terminator
	char buff[BUFFSIZE+1];
	
	//counters and index variables
	int i, inc, outc, linec, charc, bytec, bytet, percc;
	
	//this is a boolean flag that is set if the user specified
	//double precision output on the arg list
	int doubleFlagSet = 0;
	int silentFlagSet = 0;
	
	//placeholder for parsed double
	double tmpd;
	float tmpf;

	//seek and count variables
	int seek = -1, count = -1;

	//set in and out
	FILE *in, *out;

	//default in and out streams
	in = stdin;
	out = stdout;
	bytet = -1; //we dont know the length of stdin
	
	/** read user params */
	
	for(i=0; i < argc; i++){
		//check for help flag
		if(strcmp(argv[i], "help") == 0 || strcmp(argv[i], "-help") == 0){
			printf("USAGE fbinwrite [-if infile] [-of outfile] [-single] [-double] [-help] [-seek N] [-count N]\n");
			printf("\n");
			printf("Reads from stdin (or infile), searches for any numbers, and writes them to \n");
			printf("stdout (or outfile). Starts a new number each time it hits a non numeric char. \n");
			printf("\n");
			printf("Default precision is single.\n");
			printf("\n");
			return ;
		}
		//check for single
		if(strcmp(argv[i], "-single") == 0){
			doubleFlagSet = 0;
		}
		//check for double
		if(strcmp(argv[i], "-double") == 0){
			doubleFlagSet = 1;
		}
		//check for silent
		if(strcmp(argv[i], "-silent") == 0){
			silentFlagSet = 1;
		}
		//check for if
		if(strcmp(argv[i], "-if") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -if flag with no input file.\n");
				return ;
			}
			//try to open the file
			if( (in=fopen(argv[++i],"r")) == NULL ){
				printf("Error: could not open %s for reading.\n",argv[i]);
				return ;
			}
			//get length of file
			fseek(in, 0, SEEK_END);
			bytet = ftell(in);
			fseek(in, 0, SEEK_SET);
		}
		//check for of
		if(strcmp(argv[i], "-of") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -of flag with no output file.\n");
				return ;
			}
			//try to open the file
			if( (out=fopen(argv[++i],"wb")) == NULL ){
				printf("Error: could not open %s for writing.\n",argv[i]);
				return ;
			}
		}
		//check for seek
		if(strcmp(argv[i], "-seek") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -seek flag with no seek.\n");
				return ;
			}
			seek = atoi(argv[++i]);
		}
		//check for count
		if(strcmp(argv[i], "-count") == 0){
			//verify another arg is available
			if(i+1 == argc){
				printf("Error: -count flag with no count.\n");
				return ;
			}
			count = atoi(argv[++i]);
		}
	}
	
	/** end reading user params */
	
	//keep a line and char counter just for nice error messages
	inc = outc = linec = charc = bytec = percc = 0;

	//read until EOF
	i = 0;
	while( (buff[i] = fgetc(in)) != EOF){
		//increment byte counter
		bytec++;
		
		//print out a status update
		if(!silentFlagSet){
			while(bytet > 0 && bytec*20/bytet > percc){
				fprintf(stderr, "%d%% complete\n", percc*5);
				percc++;
			}
		}
	
		//increment char counter
		charc++;
		
		//increment line counter if needed
		if(buff[i] == 10 ){ 
			linec++;
			charc=0;
		}

		//completely ignore \r if it follows a \n
		if(buff[i] == 13 && charc == 1){ 
			//note that i is not incremented and char
			//is reset to zero, so we overwrite the \r
			//in the buff and reset the char pointer
			charc=0;
			continue;
		}
		
		//corresponds to 0-9 and "."
		if( (buff[i] >= 48 && buff[i] <= 57) || buff[i] == 46 ){ 
			//nothing to do here
		}
		//if the buff is empty, just ignore this invalid char
		else if(i == 0){
			continue ;
		}
		//not an acceptable float digit
		else{
			buff[i+1] = '\0';
			tmpd = atof(buff);
			tmpf = (float)tmpd;
			inc++;
			//check to see if we've already printed too many
			if(count >= 0 && outc >= count){
				break ;
			}
			//check to see if we havent skipped enough
			else if(seek >= 0 && inc <= seek){
				i=0;
			}
			//print a double
			else if(doubleFlagSet){
				fwrite(&tmpd, sizeof(double), 1, out);
				outc++;
				i=0;
			}
			//print a single
			else{
				fwrite(&tmpf, sizeof(float), 1, out);
				outc++;
				i=0;
			}
			continue ;
		}
		
		//increment index
		i++;
		
		//if i has reached the buffer size, return
		if(i == BUFFSIZE){
			fprintf(stderr, "Float too big - can not fit in buffer at line %d, char %d.",linec,charc);
			return 0;
		}
	}
	
	//good manners - close things
	if(in != stdin) fclose(in);
	if(out != stdout) fclose(out);

	//done
	if(!silentFlagSet){
		fprintf(stderr, "Done! Wrote %d floats.\n",outc);
	}
}
